MODULE eightqueensproblem;

(*   Eight Queens Solution   by      Paul Lefebvre 3-20-89

   This program will find and display all 92 ways to place 8 queens on
   a chessboard.

*)
FROM InOut IMPORT WriteInt,WriteString,WriteLn;

TYPE
   rowtype = ARRAY[1..8] OF BOOLEAN;
   sumtype = ARRAY[2..16] OF BOOLEAN;
   diftype = ARRAY[-7..7] OF BOOLEAN;
   picktype = ARRAY[1..8] OF INTEGER;
   
VAR
   rowchk : rowtype;     (* Values contain TRUE if row unavailable *)
   diagsum : sumtype;    (* Values contain TRUE if daigonal unavailable *)
   diagdif : diftype;    (* Values contain TRUE if diagonal unavailable *)
   picked : picktype;    (* Contains the row number of each column *)
   i : INTEGER;

(* ---------------------------------------------------------------------- *)

PROCEDURE displaysolution(picked : picktype; VAR solnum : INTEGER);

(* This procedure will display the solution that was found to the screen *)
VAR
   i, j : INTEGER;
BEGIN
   solnum := solnum + 1;    (* Keep a running count of the solutions *)
   WriteLn; WriteLn;
   (* This puts the actual chessboard on the screen *)
   FOR i := 1 TO 8 DO
      FOR j := 1 TO 8 DO
         IF (picked[j] = i) THEN
            WriteString('Q ');
         ELSE
            WriteString('+ ');
         END;
      END;
      WriteLn;
   END;
   WriteString('That is solution number ');  WriteInt(solnum,2);
   WriteLn;
END displaysolution; (* displaysolution *)

(* ---------------------------------------------------------------------- *)

PROCEDURE backup(VAR row, col : INTEGER; VAR rowchk : rowtype;
                  VAR diagsum : sumtype; VAR diagdif : diftype;
                  VAR picked : picktype);
                  
(* This procedure allows the computer to 'back up' if it comes to a dead
  end.  The computer is moved back one column and the marking variables are
  set back to FALSE.  The row position is moved up by one so that the
  computer does not try the same position twice.  If the row position is
  already in the last one (8), then the procedure loops again to back
  up another column. *)
  
BEGIN
   REPEAT
      col := col - 1;
      IF (col > 0) THEN 
         row := picked[col];
         picked[col] := 0;
         rowchk[row] := FALSE;
         diagsum[col+row] := FALSE;
         diagdif[col-row] := FALSE;
   
         IF (row < 8) THEN
            row := row + 1;
         ELSE
            row := 99;
         END;
      END;
    UNTIL (row <= 8) OR (col = 0);
END backup; (* backup *)

(* ---------------------------------------------------------------------- *)

PROCEDURE controlbackups(VAR row, col : INTEGER; VAR rowchk : rowtype;
                           VAR diagsum : sumtype; VAR diagdif : diftype;
                           VAR picked : picktype; VAR found : BOOLEAN;
                           VAR solnum : INTEGER);
                           
(* This is the procedure that determines if a solution has been found or
  if the computer is stuck and should back up. *)
  

BEGIN

   (* Solution found! *)

   IF (col = 8) AND found THEN
      displaysolution(picked, solnum);
      found := FALSE;
      col := 9;
      backup(row, col, rowchk, diagsum, diagdif, picked);
   ELSE

      (* Solution not found, so back up *)

      IF (row > 8) THEN
         backup(row, col, rowchk, diagsum, diagdif, picked);
      END;
   END;
END controlbackups; (* controlbackups *)

(* ---------------------------------------------------------------------- *)

PROCEDURE findsolutions(rowchk : rowtype; diagsum : sumtype;
                         diagdif : diftype; picked : picktype);
                         
(* This is the procedure that does the actual checking for a correct solution.
  It will return to the main body only when all the solutions are found. *)
  
VAR
   found, done : BOOLEAN;
   start, solnum, row, col : INTEGER;
   
BEGIN
   start := 1;
   picked[1] := start;
   rowchk[start] := TRUE;
   diagsum[1+start] := TRUE;
   diagdif[1-start] := TRUE;
   done := FALSE;
   solnum := 0;
   col := 2;
   REPEAT
      row := 1;
      REPEAT
         found := FALSE;
         IF (col > 0) THEN
            IF (NOT rowchk[row]) AND (NOT diagsum[col+row]) AND
               (NOT diagdif[col-row]) THEN
               
               found := TRUE;
               rowchk[row] := TRUE;
               diagsum[col+row] := TRUE;
               diagdif[col-row] := TRUE;
               picked[col] := row;
            
            ELSE
               row := row + 1;
            END;
            controlbackups(row, col, rowchk, diagsum, diagdif, picked,
                             found, solnum);
         ELSE
            found := TRUE;
            done := TRUE;
         END
      UNTIL found;
      col := col + 1;
   UNTIL done;
END findsolutions;(* findsolutions *)
   
(* ====================================================================== *)

BEGIN (* Main body *)

   (* Make sure all the arrays are correctly initialized *)

   FOR i := 1 TO 8 DO
      rowchk[i] := FALSE;
      picked[i] := 0;
   END;
   FOR i := 2 TO 16 DO
      diagsum[i] := FALSE;
   END;
   FOR i := -7 TO 7 DO
      diagdif[i] := FALSE;
   END;

   WriteString('Working ...');
   WriteLn;
   WriteLn;
   
   findsolutions(rowchk,diagsum,diagdif,picked);

   WriteLn;
   WriteString('That is all folks!');  (* Program takes approx. 41 seconds to get here *)
   WriteLn;
END eightqueensproblem.

